#include<Joystick.h>

const int numBoards = 8; // the number of different ground lines being used
const int buttonsPerBoard = 5; // the number of buttons on each ground line

// the joystick object (has only buttons, the number of buttons is numBoards*buttonsPerBoard)
Joystick_ Joystick(JOYSTICK_DEFAULT_REPORT_ID, JOYSTICK_TYPE_JOYSTICK, numBoards*buttonsPerBoard, 0, false, false, false, false, false, false, false, false, false, false, false);

int buttons[buttonsPerBoard] = {5, 6, 7, 8, 9};
int boards[numBoards] = {A3, A2, A1, A0, 15, 14, 16, 10};

int joyButtonLastStates[numBoards*buttonsPerBoard] = {0};

void setup() {
  // make each board pin an output pin
  for (int i=0; i<numBoards; i++) {
    pinMode(boards[i], OUTPUT);
  }

  delay(10);

  // set each board pin to HIGH (we only want the board we're reading to be set to LOW, all the others should be HIGH)
  for (int i=0; i<numBoards; i++) {
    digitalWrite(boards[i], HIGH);
  }

  // make each button pin an input pin with the pullup resistor used
  for (int i=0; i<buttonsPerBoard; i++) {
    pinMode(buttons[i], INPUT_PULLUP);
  }

  // start being a joystick
  Joystick.begin();
  
  delay(10);
}

void loop() {
  for (int i=0; i<numBoards; i++) { // for each board...
    digitalWrite(boards[i], LOW);   // ground that board's ground line (instead of it being HIGH)
    
    for (int j=0; j<buttonsPerBoard; j++) { // then loop through each button, checking to see if it's state has changed
      int currentState = !digitalRead(buttons[j]);

      if (currentState != joyButtonLastStates[i*buttonsPerBoard+j]) { // if that button's state has changed...
        Joystick.setButton(i*buttonsPerBoard+j, currentState);        // adjust the Joystick object's button to match the physical button
        joyButtonLastStates[i*buttonsPerBoard+j] = currentState;      // remember that the button is now in its current state for next time the loop runs
      }
    }

    digitalWrite(boards[i], HIGH); // set that board's ground rail back to HIGH
  }
}
